<?php

namespace Drupal\monarch_migration_tools\Plugin\migrate\destination;

use Drupal\migrate\Row;
use Drupal\migrate\MigrateException;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\migrate\Plugin\migrate\destination\DestinationBase;

/**
 * Imports a config as simply as possbile.
 *
 * This allow us to sequence custom configuration imports as part of a
 * complex migration setup.
 *
 * @MigrateDestination(
 *   id = "import_config"
 * )
 */
class ImportConfig extends DestinationBase implements ContainerFactoryPluginInterface {

  /**
   * The 'config.factory' service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   * */
  protected $configFactory;

  /**
   * Constructs a new ImportConfig.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\migrate\Plugin\MigrationInterface $migration
   *   The migration.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration);
    $this->supportsRollback = FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration = NULL) {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $migration,
    );

    $instance->configFactory = $container->get('config.factory');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    return [
      'name' => [
        'type' => 'string',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function fields(MigrationInterface $migration = NULL) {
    return [
      'name' => $this->t('Configuration Name'),
      'data' => $this->t('Configuration Data'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function import(Row $row, array $old_destination_id_values = []) {
    $name = $row->getDestinationProperty('name');
    $data = $row->getDestinationProperty('data');

    if (empty($name)) {
      throw new MigrateException('Imported config must have a name.');
    }

    try {

      if (isset($data['uuid'])) {
        unset($data['uuid']);
      }

      $config = $this->configFactory->getEditable($name);
      $config->setData($data)->save(TRUE);

      return [$name];
    }
    catch (\Throwable $e) {
      throw new MigrateException($e->getMessage());
    }
  }

}
